<?php

namespace Drupal\editor_note\Form;

use Drupal\editor_note\Entity\EditorNote;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\RemoveCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class ConfirmDeleteEditorNoteForm.
 */
class ConfirmDeleteEditorNoteForm extends FormBase {

  /**
   * Editor Note entity.
   *
   * @var \Drupal\editor_note\Entity\EditorNote
   */
  protected $editorNote;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'confirm_delete_editor_note_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, EditorNote $editor_note = NULL, $nojs = NULL) {
    $this->editorNote = $editor_note;

    $form['use_ajax_container']['description'] = [
      '#type' => 'item',
      '#markup' => $this->t('Are you sure you want to remove the note? This action cannot be undone.'),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
      '#attributes' => [
        'class' => [
          'use-ajax',
        ],
      ],
      '#ajax' => [
        'callback' => [$this, 'submitModalFormAjax'],
        'event' => 'click',
      ],
    ];

    $form['cancel'] = [
      '#type' => 'button',
      '#value' => $this->t('Cancel'),
      '#attributes' => [
        'class' => [
          'use-ajax',
        ],
      ],
      '#ajax' => [
        'callback' => [$this, 'closeModalForm'],
        'event' => 'click',
      ],
    ];

    return $form;
  }

  /**
   * Ajax callback for the "Submit" button.
   *
   * Remove table row and close modal dialog.
   *
   * @param array $form
   *   The Form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   Ajax response.
   */
  public function submitModalFormAjax(array $form, FormStateInterface $form_state) {
    $response = new AjaxResponse();

    if ($this->editorNote) {
      $selector = '.note-' . $this->editorNote->id();
      $removeCommand = new RemoveCommand($selector);
      $response->addCommand($removeCommand);
    }
    $command = new CloseModalDialogCommand();
    $response->addCommand($command);

    return $response;
  }

  /**
   * Ajax callback for the "Cancel" button.
   *
   * Close modal dialog.
   *
   * @param array $form
   *   The Form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   Ajax response.
   */
  public function closeModalForm(array $form, FormStateInterface $form_state) {
    $response = new AjaxResponse();
    $command = new CloseModalDialogCommand();
    $response->addCommand($command);
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Remove Editor Note entity.
    if ($this->editorNote) {
      $this->editorNote->delete();
    }
  }

}
